#ifndef __BPPLIB_HPP
#define __BPPLIB_HPP

#include "AEEStdlib.h"
#include "AEEGraphics.h"


extern void* operator new(size_t size) throw();
extern void* operator new(size_t size, void* address) throw();
extern void* operator new [](size_t size) throw();
extern void* operator new [](size_t size, void* address) throw();
extern void operator delete(void* address) throw();
extern void operator delete(void* address, void* dummy) throw();
extern void operator delete [](void* address) throw();
extern void operator delete [](void* address, void* dummy) throw();

class BPPString;

extern void DrawText(IDisplay* disp, const BPPString& string, int x, int y);
extern void DrawRect(IDisplay* disp, int x, int y, int dx, int dy, RGBVAL color);
extern void FillRect(IDisplay* disp, int x, int y, int dx, int dy, RGBVAL color);

class BPPHeap {
private:
	void*			m_heap;
	unsigned int	m_size;
public:
	BPPHeap();
	~BPPHeap();
	operator void*() const { return m_heap; }
	void Free();
	boolean Copy(const void* heap, unsigned int size);
	boolean Resize(unsigned int size);
	const void* GetHeap() const { return m_heap; }
	void* GetHeap() { return m_heap; }
	unsigned int GetSize() const { return m_size; }
};

typedef char AChar ;

class BPPString {
private:
	BPPHeap m_string;
	unsigned int m_length;
	boolean m_error;
	boolean SetHeapSize(unsigned int size);
public:
	BPPString();
	BPPString(const BPPString& string);
	BPPString(const AChar* string, int length = -1);
	unsigned int GetLength() const { return m_length; }
	boolean SetLength(unsigned int length);
	boolean Copy(unsigned int index, const BPPString &string);
	boolean Copy(unsigned int index, const AChar* string, int length = -1);
	boolean Concat(const BPPString &string) { return Copy(m_length, string); }
	boolean Concat(const AChar* string, int length = -1) { return Copy(m_length, string, length); }
	boolean Shrink() { return SetHeapSize(m_length + 1); }
	int Compare(const BPPString& string) const { return Compare((const AChar*)string.m_string.GetHeap()); }
	int Compare(const AChar* string) const { return STRCMP((const AChar*)m_string.GetHeap(), string); }
	void Clear();
	boolean GetError() const { return m_error; }
	void ClearError() { m_error = FALSE; }
	AChar* GetBuffer() { return (AChar*)m_string.GetHeap(); }
	const AChar* GetBuffer() const { return (const AChar*)m_string.GetHeap(); }
	boolean IsEmpty() const { return m_length == 0; }
	BPPString Substring(unsigned int begin, unsigned int end) const { return BPPString((const AChar*)m_string.GetHeap() + begin, end - begin); }
	static BPPString IntToString(int value);
#ifdef AEE_SIMULATOR
	AChar& operator [](unsigned int index);
	const AChar& operator [](unsigned int index) const;
#else
	AChar& operator [](unsigned int index) { return *((AChar*)m_string.GetHeap() + index); }
	const AChar& operator [](unsigned int index) const { return *((AChar*)m_string.GetHeap() + index); }
#endif
	BPPString& operator =(const BPPString& string) {
		Copy(0, string);
		return *this;
	}
	BPPString& operator =(const AChar* string) {
		Copy(0, string);
		return *this;
	}
	BPPString& operator +=(const BPPString& string) {
		Concat(string);
		return *this;
	}
	BPPString& operator +=(const AChar* string) {
		Concat(string);
		return *this;
	}

};

// Zq̑O錾
boolean operator ==(const BPPString& string1, const BPPString& string2);
boolean operator ==(const BPPString& string1, const AChar* string2);
boolean operator ==(const AChar* string1, const BPPString& string2);
boolean operator !=(const BPPString& string1, const BPPString& string2);
boolean operator !=(const BPPString& string1, const AChar* string2);
boolean operator !=(const AChar* string1, const BPPString& string2);
boolean operator <=(const BPPString& string1, const BPPString& string2);
boolean operator <=(const BPPString& string1, const AChar* string2);
boolean operator <=(const AChar* string1, const BPPString& string2);
boolean operator <(const BPPString& string1, const BPPString& string2);
boolean operator <(const BPPString& string1, const AChar* string2);
boolean operator <(const AChar* string1, const BPPString& string2);
boolean operator >=(const BPPString& string1, const BPPString& string2);
boolean operator >=(const BPPString& string1, const AChar* string2);
boolean operator >=(const AChar* string1, const BPPString& string2);
boolean operator >(const BPPString& string1, const BPPString& string2);
boolean operator >(const BPPString& string1, const AChar* string2);
boolean operator >(const AChar* string1, const BPPString& string2);
BPPString operator +(const BPPString& string1, const BPPString& string2);
BPPString operator +(const BPPString& string1, const AChar* string2);
BPPString operator +(const AChar* string1, const BPPString& string2);

inline boolean operator ==(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : !string1.Compare(string2);
}

inline boolean operator ==(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : !string1.Compare(string2);
}

inline boolean operator ==(const AChar* string1, const BPPString& string2)
{
	return operator ==(string2, string1);
}

inline boolean operator !=(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : string1.Compare(string2);
}

inline boolean operator !=(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : string1.Compare(string2);
}

inline boolean operator !=(const AChar* string1, const BPPString& string2)
{
	return operator !=(string2, string1);
}

inline boolean operator <=(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : (string1.Compare(string2) <= 0);
}

inline boolean operator <=(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : (string1.Compare(string2) <= 0);
}

inline boolean operator <=(const AChar* string1, const BPPString& string2)
{
	return operator >=(string2, string1);
}

inline boolean operator <(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : (string1.Compare(string2) < 0);
}

inline boolean operator <(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : (string1.Compare(string2) < 0);
}

inline boolean operator <(const AChar* string1, const BPPString& string2)
{
	return operator >(string2, string1);
}

inline boolean operator >=(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : (string1.Compare(string2) >= 0);
}

inline boolean operator >=(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : (string1.Compare(string2) >= 0);
}

inline boolean operator >=(const AChar* string1, const BPPString& string2)
{
	return operator <=(string2, string1);
}

inline boolean operator >(const BPPString& string1, const BPPString& string2)
{
	return (string1.GetError() || string2.GetError()) ? FALSE : (string1.Compare(string2) > 0);
}

inline boolean operator >(const BPPString& string1, const AChar* string2)
{
	return string1.GetError() ? FALSE : (string1.Compare(string2) > 0);
}

inline boolean operator >(const AChar* string1, const BPPString& string2)
{
	return operator <(string2, string1);
}

inline BPPString operator +(const BPPString& string1, const BPPString& string2)
{
	return BPPString(string1).operator +=(string2);
}

inline BPPString operator +(const BPPString& string1, const AChar* string2)
{
	return BPPString(string1).operator +=(string2);
}

inline BPPString operator +(const AChar* string1, const BPPString& string2)
{
	return BPPString(string1).operator +=(string2);
}

#endif //__BPPLIB_HPP