#include "BPPLib.hpp"

void* operator new(size_t size) throw()
{
    return MALLOC(size);
}

void* operator new(size_t /*size*/, void* address) throw()
{
    return address;
}

void* operator new [](size_t size) throw()
{
    return MALLOC(size);
}

void* operator new [](size_t /*size*/, void* address) throw()
{
    return address;
}

void operator delete(void* address) throw()
{
    FREE(address);
    return;
}

void operator delete(void* /*address*/, void* /*dummy*/) throw()
{
    return;
}

void operator delete [](void* address) throw()
{
    FREE(address);
    return;
}

void operator delete [](void* /*address*/, void* /*dummy*/) throw()
{
    return;
}

BPPHeap::BPPHeap() : m_heap(NULL), m_size(0) {
	return;
}

BPPHeap::~BPPHeap() {
	Free();
	return;
}

void BPPHeap::Free() {
	if (m_heap != NULL) {
		FREE(m_heap);
		m_heap = NULL;
	}
	m_size = 0;
	return;
}

boolean BPPHeap::Copy(const void* heap, unsigned int size) {
	boolean ret = Resize(size);
	if (ret) {
		MEMCPY(m_heap, heap, size);
	}
	return ret;
}

boolean BPPHeap::Resize(unsigned int size) {
	void* temp;
	if (m_heap == NULL) {
		m_heap = MALLOC(size);
		if (m_heap == NULL) {
			return FALSE;
		}
	}
	else {
		if (m_size != size) {
			if (size == 0)
				Free();
			else {
				temp = REALLOC(m_heap, size);
				if (temp == NULL)
					return FALSE;
				m_heap = temp;
				m_size = size;
			}
		}
	}
	return TRUE;
}

#define INTEGER_DIGITS	11

BPPString::BPPString() : m_length(0), m_error(FALSE)
{
	return;
}

BPPString::BPPString(const BPPString& string) : m_length(0), m_error(FALSE)
{
	Copy(0, string);
	return;
}

BPPString::BPPString(const AChar* string, int length) : m_length(0), m_error(FALSE)
{
	Copy(0, string, length);
	return;
}

boolean BPPString::SetHeapSize(unsigned int size)
{
	boolean ret = TRUE;
	if (m_string.GetSize() < size) {
		ret = m_string.Resize(size);
		if(!ret)
			m_error = TRUE;
	}
	return ret;
}

boolean BPPString::SetLength(unsigned int length)
{
	boolean ret(SetHeapSize(length + 1));
	if (ret)
		*((AChar*)m_string.GetHeap() + (m_length = length)) = '\0';
	return ret;
}

boolean BPPString::Copy(unsigned int index, const BPPString &string)
{
	if (!string.m_error)
		m_error = string.m_error;
	return Copy(index, (const AChar*)string.m_string.GetHeap(), string.m_length);
}
boolean BPPString::Copy(unsigned int index, const AChar* string, int length)
{
	if (!string)
		length = 0;
	if (length < 0)
		length = STRLEN(string);
	unsigned int newlength = index + length;
	if (!SetHeapSize(newlength + 1))
		return FALSE;
	MEMCPY((AChar*)m_string.GetHeap() + index, string, length);
	*((AChar*)m_string.GetHeap() + (m_length = newlength)) = '\0';
	return TRUE;
}

void BPPString::Clear()
{
	m_string.Free();
	m_length = 0;
	return;
}

BPPString BPPString::IntToString(int value)
{
	BPPString ret;
	if (ret.SetLength(INTEGER_DIGITS))
		SPRINTF((AChar*)ret.m_string.GetHeap(), "%d", value);
	return ret;
}

#ifdef AEE_SUMULATOR
AChar& BPPString::operator [](unsigned int index)
{
	if (index > m_length)
		DBGPRINTF("Invalid index! (file = %s, line = %d)", __FILE__, __LINE__);
	return *((AChar*)m_string.GetHeap() + index);
}

const AChar BPPString::operator [](unsigned int index) const
{
	if (index > m_length)
		DBGPRINTF("Invalid index! (file = %s, line = %d)", __FILE__, __LINE__);
	return *((AChar*)m_string.GetHeap() + index);
}

#endif

void DrawText(IDisplay* disp, const BPPString& string, int x, int y) {
    if (string.IsEmpty()) return;
    AECHAR* wstr;
    int wstr_bytes;

    wstr_bytes = (string.GetLength() + 1) * sizeof(AECHAR);
    wstr = (AECHAR*) MALLOC(wstr_bytes);

    STREXPAND((const byte*)string.GetBuffer(), string.GetLength(), wstr, wstr_bytes);

    // ʃrbg}bvɕ\
    IDISPLAY_DrawText(disp,
        AEE_FONT_NORMAL,
        wstr,             // \镶
        -1,               // -1 = ׂĕ\
        x,                // xW
        y,                // yW
        NULL,             // NbsOȂ
        0);   // 

    FREE(wstr);
}


void DrawRect(IDisplay* disp, int x, int y, int dx, int dy, RGBVAL color) {
    AEERect rect = {x, y, dx, dy};
    IDISPLAY_DrawRect(disp, &rect, color, 0, IDF_RECT_FRAME);
}


void FillRect(IDisplay* disp, int x, int y, int dx, int dy, RGBVAL color) {
    AEERect rect = {x, y, dx, dy};
    IDISPLAY_DrawRect(disp, &rect, color, color, IDF_RECT_FILL);
}
