#include "LifeGame.hpp"
#include "LifeGameApp.hpp"

LifeGame::LifeGame() {
    width  = MATRIX_WIDTH;
    height  = MATRIX_HEIGHT;
    current = 0;
    matrix[0].init(width + 2, height + 2);
    matrix[1].init(width + 2, height + 2);

    matrix[0].setCell(20, 26, 1);
    matrix[0].setCell(21, 27, 1);
    matrix[0].setCell(25, 27, 1);
    matrix[0].setCell(26, 25, 1);
    matrix[0].setCell(26, 27, 1);
    matrix[0].setCell(27, 27, 1);
}

Matrix& LifeGame::getMatrix(){
    return matrix[current];
}

void LifeGame::setSize(int width_, int height_){
    Matrix m;
    m.init(width_ + 2, height_ + 2);
    int minw = (width < width_) ? width : width_;
    int minh = (height < height_) ? height : height_;
    width = width_;
    height = height_;

    for(int i=0;i<minw;i++){
        for(int j=0;j<minh;j++){
            m.setCell(i, j, matrix[current].getCell(i, j));
        }
    }
    matrix[current] = m;
    matrix[(current == 0) ? 1 : 0].init(width + 2, height + 2);
}

void LifeGame::next(){
    DBGPRINTF("timer");
    int next = (current == 0) ? 1 : 0;
    int life;

    for(int i=1;i<width+1;i++){
        for(int j=1;j<height+1;j++){
            matrix[next].setCell(i, j, 0);
            life = 0;
            for(int a=i-1;a<=i+1;a++){
                for (int b = j - 1; b <= j + 1; b++) {
                    if(a == i && b == j){
                        continue;
                    }
                    if (matrix[current].getCell(a, b)) {
                        life++;
                    }
                }
            }
            if(matrix[current].getCell(i, j)){
                if(1 < life && life <= 3){
                    matrix[next].setCell(i, j, 1);
                }
            } else{
                if(life == 3){
                    matrix[next].setCell(i, j, 1);
                }
            }
        }
    }
    current = next;
}

void LifeGame::setMatrix(Matrix& copy) {
    matrix[current] = copy;
    width = copy.getWidth() - 2;
    height = copy.getHeight() - 2;
    matrix[(current == 0) ? 1 : 0].init(width + 2, height + 2);
}

void LifeGame::paint(IDisplay* display) {
    Matrix& m = matrix[current];
    DrawRect(display,
        CELL_WIDTH-1, CELL_HEIGHT-1, width*CELL_WIDTH, height*CELL_HEIGHT,
        MAKE_RGB(255, 0, 0));

    for(int i=1;i<width+1;i++){
        for(int j=1;j<height+1;j++){
            if(m.getCell(i, j)){
                FillRect(display,
                    i * CELL_WIDTH, j * CELL_HEIGHT, CELL_WIDTH-1, CELL_HEIGHT-1,
                    MAKE_RGB(0, 0, 255));
            }
        }
    }
}
